@@ -1,6 +1,8 @@
 """Provide a model for the Z-Wave JS node."""
 from __future__ import annotations
 
+import asyncio
+import copy
 import logging
 from datetime import datetime
 from typing import TYPE_CHECKING, Any, cast
@@ -9,6 +11,7 @@ from ...const import (
     INTERVIEW_FAILED,
     NOT_INTERVIEWED,
     CommandClass,
+    DateAndTime,
     NodeStatus,
     PowerLevel,
     SecurityClass,
@@ -37,6 +40,7 @@ from ..notification import (
 from ..value import (
     ConfigurationValue,
     MetaDataType,
+    SetValueResult,
     Value,
     ValueDataType,
     ValueMetadata,
@@ -56,7 +60,9 @@ from .firmware import (
 )
 from .health_check import (
     CheckHealthProgress,
+    LifelineHealthCheckResult,
     LifelineHealthCheckSummary,
+    RouteHealthCheckResult,
     RouteHealthCheckSummary,
     TestPowerLevelProgress,
 )
@@ -66,6 +72,7 @@ if TYPE_CHECKING:
     from ...client import Client
 
 
+# pylint: disable=too-many-lines
 _LOGGER = logging.getLogger(__package__)
 
 DEFAULT_NODE_STATISTICS = NodeStatisticsDataType(
@@ -107,6 +114,7 @@ class Node(EventBase):
         self._firmware_update_progress: NodeFirmwareUpdateProgress | None = None
         self.values: dict[str, ConfigurationValue | Value] = {}
         self.endpoints: dict[int, Endpoint] = {}
+        self._status_event = asyncio.Event()
         self.update(data)
 
     def __repr__(self) -> str:
@@ -190,9 +198,9 @@ class Node(EventBase):
     @property
     def is_secure(self) -> bool | None:
         """Return the is_secure."""
-        if (is_secure := self.data.get("isSecure")) == "unknown":
-            return None
-        return is_secure
+        if (is_secure := self.data.get("isSecure")) is not None:
+            return is_secure
+        return None
 
     @property
     def protocol_version(self) -> int | None:
@@ -345,26 +353,53 @@ class Node(EventBase):
         """Return whether the node is set to keep awake."""
         return self.data["keepAwake"]
 
+    @property
+    def last_seen(self) -> datetime | None:
+        """Return when the node was last seen."""
+        if last_seen := self.data.get("lastSeen"):
+            return datetime.fromisoformat(last_seen)
+        return None
+
+    @property
+    def default_volume(self) -> int | float | None:
+        """Return the default volume."""
+        return self.data.get("defaultVolume")
+
+    @property
+    def default_transition_duration(self) -> int | float | None:
+        """Return the default transition duration."""
+        return self.data.get("defaultTransitionDuration")
+
     def update(self, data: NodeDataType) -> None:
         """Update the internal state data."""
-        self.data = data
+        self.data = copy.deepcopy(data)
         self._device_config = DeviceConfig(self.data.get("deviceConfig", {}))
         self._statistics = NodeStatistics(
             self.client, self.data.get("statistics", DEFAULT_NODE_STATISTICS)
         )
 
-        # Remove stale values
-        value_ids = (_get_value_id_str_from_dict(self, val) for val in data["values"])
-        self.values = {
-            value_id: val
-            for value_id, val in self.values.items()
-            if value_id in value_ids
+        new_values_data = {
+            _get_value_id_str_from_dict(self, val): val
+            for val in self.data.pop("values")
         }
+        new_value_ids = set(new_values_data)
+        stale_value_ids = set(self.values) - new_value_ids
 
-        # Populate new values
-        for val in data["values"]:
+        # Remove stale values
+        for value_id in stale_value_ids:
+            self.values.pop(value_id)
+
+        # Updating existing values and populate new values. Preserve value order if
+        # initializing values for the node for the first time by using the key order
+        # which is deterministic
+        for value_id in (
+            new_value_ids - stale_value_ids
+            if stale_value_ids
+            else list(new_values_data)
+        ):
+            val = new_values_data[value_id]
             try:
-                if (value_id := _get_value_id_str_from_dict(self, val)) in self.values:
+                if value_id in self.values:
                     self.values[value_id].update(val)
                 else:
                     self.values[value_id] = _init_value(self, val)
@@ -372,29 +407,28 @@ class Node(EventBase):
                 # If we can't parse the value, don't store it
                 pass
 
-        # Remove stale endpoints
-        self.endpoints = {
-            idx: endpoint
-            for idx, endpoint in self.endpoints.items()
-            if idx in (endpoint["index"] for endpoint in self.data["endpoints"])
+        new_endpoints_data = {
+            endpoint["index"]: endpoint for endpoint in self.data.pop("endpoints")
         }
+        new_endpoint_idxs = set(new_endpoints_data)
+        stale_endpoint_idxs = set(self.endpoints) - new_endpoint_idxs
+
+        # Remove stale endpoints
+        for endpoint_idx in stale_endpoint_idxs:
+            self.endpoints.pop(endpoint_idx)
 
         # Add new endpoints or update existing ones
-        for endpoint in self.data["endpoints"]:
-            idx = endpoint["index"]
+        for endpoint_idx in new_endpoint_idxs - stale_endpoint_idxs:
+            endpoint = new_endpoints_data[endpoint_idx]
             values = {
                 value_id: value
                 for value_id, value in self.values.items()
                 if self.index == value.endpoint
             }
-            if idx in self.endpoints:
-                self.endpoints[idx].update(endpoint, values)
+            if endpoint_idx in self.endpoints:
+                self.endpoints[endpoint_idx].update(endpoint, values)
             else:
-                self.endpoints[idx] = Endpoint(
-                    self.client,
-                    endpoint,
-                    values,
-                )
+                self.endpoints[endpoint_idx] = Endpoint(self.client, endpoint, values)
 
     def get_command_class_values(
         self, command_class: CommandClass, endpoint: int | None = None
@@ -441,11 +475,26 @@ class Node(EventBase):
         if require_schema is not None:
             kwargs["require_schema"] = require_schema
 
-        if wait_for_result or (
-            wait_for_result is None and self.status != NodeStatus.ASLEEP
-        ):
+        if wait_for_result:
             result = await self.client.async_send_command(message, **kwargs)
             return result
+        if wait_for_result is None and self.status not in (
+            NodeStatus.ASLEEP,
+            NodeStatus.DEAD,
+        ):
+            result_task = asyncio.create_task(
+                self.client.async_send_command(message, **kwargs)
+            )
+            status_task = asyncio.create_task(self._status_event.wait())
+            await asyncio.wait(
+                [result_task, status_task],
+                return_when=asyncio.FIRST_COMPLETED,
+            )
+            status_task.cancel()
+            if self._status_event.is_set() and not result_task.done():
+                result_task.cancel()
+                return None
+            return result_task.result()
 
         await self.client.async_send_command_no_wait(message, **kwargs)
         return None
@@ -456,7 +505,7 @@ class Node(EventBase):
         new_value: Any,
         options: dict | None = None,
         wait_for_result: bool | None = None,
-    ) -> bool | None:
+    ) -> SetValueResult | None:
         """Send setValue command to Node for given value (or value_id)."""
         # a value may be specified as value_id or the value itself
         if not isinstance(val, Value):
@@ -488,13 +537,13 @@ class Node(EventBase):
 
         # the value object needs to be send to the server
         result = await self.async_send_command(
-            "set_value", **cmd_args, wait_for_result=wait_for_result
+            "set_value", **cmd_args, require_schema=29, wait_for_result=wait_for_result
         )
 
         if result is None:
             return None
 
-        return cast(bool, result["success"])
+        return SetValueResult(result["result"])
 
     async def async_refresh_info(self) -> None:
         """Send refreshInfo command to Node."""
@@ -572,7 +621,7 @@ class Node(EventBase):
 
     async def async_abort_firmware_update(self) -> None:
         """Send abortFirmwareUpdate command to Node."""
-        await self.async_send_command("abort_firmware_update", wait_for_result=True)
+        await self.async_send_command("abort_firmware_update", wait_for_result=False)
 
     async def async_poll_value(self, val: Value | str) -> None:
         """Send pollValue command to Node for given value (or value_id)."""
@@ -631,7 +680,9 @@ class Node(EventBase):
         """Call endpoint.get_node_unsafe command."""
         return await self.endpoints[0].async_get_node_unsafe()
 
-    async def async_has_security_class(self, security_class: SecurityClass) -> bool:
+    async def async_has_security_class(
+        self, security_class: SecurityClass
+    ) -> bool | None:
         """Return whether node has the given security class."""
         data = await self.async_send_command(
             "has_security_class",
@@ -639,16 +690,18 @@ class Node(EventBase):
             require_schema=8,
             wait_for_result=True,
         )
-        assert data
-        return cast(bool, data["hasSecurityClass"])
+        if data and (has_security_class := data.get("hasSecurityClass")) is not None:
+            return cast(bool, has_security_class)
+        return None
 
-    async def async_get_highest_security_class(self) -> SecurityClass:
+    async def async_get_highest_security_class(self) -> SecurityClass | None:
         """Get the highest security class that a node supports."""
         data = await self.async_send_command(
             "get_highest_security_class", require_schema=8, wait_for_result=True
         )
-        assert data
-        return SecurityClass(data["highestSecurityClass"])
+        if data and (security_class := data.get("highestSecurityClass")) is not None:
+            return SecurityClass(security_class)
+        return None
 
     async def async_test_power_level(
         self, test_node: "Node", power_level: PowerLevel, test_frame_count: int
@@ -802,7 +855,9 @@ class Node(EventBase):
             wait_for_result=False,
         )
 
-    async def async_set_date_and_time(self, datetime_: datetime | None = None) -> bool:
+    async def async_set_date_and_time(
+        self, datetime_: datetime | None = None, wait_for_result: bool | None = None
+    ) -> bool | None:
         """Send setDateAndTime command to Node."""
         args = {}
         if datetime_:
@@ -811,10 +866,80 @@ class Node(EventBase):
             "set_date_and_time",
             **args,
             require_schema=28,
+            wait_for_result=wait_for_result,
+        )
+        if data:
+            return cast(bool, data["success"])
+        return None
+
+    async def async_get_date_and_time(self) -> DateAndTime:
+        """Send getDateAndTime command to Node."""
+        data = await self.async_send_command(
+            "get_date_and_time",
+            require_schema=31,
             wait_for_result=True,
         )
         assert data
-        return cast(bool, data["success"])
+        return DateAndTime(data["dateAndTime"])
+
+    async def async_is_health_check_in_progress(self) -> bool:
+        """Send isHealthCheckInProgress command to Node."""
+        data = await self.async_send_command(
+            "is_health_check_in_progress",
+            require_schema=31,
+            wait_for_result=True,
+        )
+        assert data
+        return cast(bool, data["progress"])
+
+    async def async_abort_health_check(self) -> None:
+        """Send abortHealthCheck command to Node."""
+        await self.async_send_command(
+            "abort_health_check",
+            require_schema=31,
+            wait_for_result=False,
+        )
+
+    async def async_set_default_volume(
+        self, default_volume: int | float | None
+    ) -> None:
+        """Send setDefaultVolume command to Node."""
+        cmd_kwargs = {}
+        self.data["defaultVolume"] = default_volume
+        if default_volume is not None:
+            cmd_kwargs["defaultVolume"] = default_volume
+        await self.async_send_command(
+            "set_default_volume",
+            require_schema=31,
+            wait_for_result=None,
+            **cmd_kwargs,
+        )
+
+    async def async_set_default_transition_duration(
+        self, default_duration_transition: int | float | None
+    ) -> None:
+        """Send setDefaultTransitionDuration command to Node."""
+        cmd_kwargs = {}
+        self.data["defaultTransitionDuration"] = default_duration_transition
+        if default_duration_transition is not None:
+            cmd_kwargs["defaultTransitionDuration"] = default_duration_transition
+        await self.async_send_command(
+            "set_default_transition_duration",
+            require_schema=31,
+            wait_for_result=None,
+            **cmd_kwargs,
+        )
+
+    async def async_has_device_config_changed(self) -> bool | None:
+        """Send hasDeviceConfigChanged command to Node."""
+        data = await self.async_send_command(
+            "has_device_config_changed",
+            require_schema=31,
+            wait_for_result=True,
+        )
+        if data and (changed := data.get("changed")) is not None:
+            return cast(bool, changed)
+        return None
 
     def handle_test_powerlevel_progress(self, event: Event) -> None:
         """Process a test power level progress event."""
@@ -825,33 +950,43 @@ class Node(EventBase):
     def handle_check_lifeline_health_progress(self, event: Event) -> None:
         """Process a check lifeline health progress event."""
         event.data["check_lifeline_health_progress"] = CheckHealthProgress(
-            event.data["rounds"], event.data["totalRounds"], event.data["lastRating"]
+            event.data["rounds"],
+            event.data["totalRounds"],
+            event.data["lastRating"],
+            LifelineHealthCheckResult(event.data["lastResult"]),
         )
 
     def handle_check_route_health_progress(self, event: Event) -> None:
         """Process a check route health progress event."""
         event.data["check_route_health_progress"] = CheckHealthProgress(
-            event.data["rounds"], event.data["totalRounds"], event.data["lastRating"]
+            event.data["rounds"],
+            event.data["totalRounds"],
+            event.data["lastRating"],
+            RouteHealthCheckResult(event.data["lastResult"]),
         )
 
     def handle_wake_up(self, event: Event) -> None:
         """Process a node wake up event."""
         # pylint: disable=unused-argument
+        self._status_event.clear()
         self.data["status"] = NodeStatus.AWAKE
 
     def handle_sleep(self, event: Event) -> None:
         """Process a node sleep event."""
         # pylint: disable=unused-argument
+        self._status_event.set()
         self.data["status"] = NodeStatus.ASLEEP
 
     def handle_dead(self, event: Event) -> None:
         """Process a node dead event."""
         # pylint: disable=unused-argument
+        self._status_event.set()
         self.data["status"] = NodeStatus.DEAD
 
     def handle_alive(self, event: Event) -> None:
         """Process a node alive event."""
         # pylint: disable=unused-argument
+        self._status_event.clear()
         self.data["status"] = NodeStatus.ALIVE
 
     def handle_interview_started(self, event: Event) -> None:
@@ -883,15 +1018,6 @@ class Node(EventBase):
         """Process a node value added event."""
         self.handle_value_updated(event)
 
-    def value_data_idx(self, value_id: str) -> int:
-        """Get the index for the given value ID in the node's value data."""
-        values = self.data["values"]
-        return next(
-            idx
-            for idx in range(len(values))
-            if _get_value_id_str_from_dict(self, values[idx]) == value_id
-        )
-
     def handle_value_updated(self, event: Event) -> None:
         """Process a node value updated event."""
         evt_val_data: ValueDataType = event.data["args"]
@@ -900,24 +1026,14 @@ class Node(EventBase):
         if value is None:
             value = _init_value(self, evt_val_data)
             self.values[value.value_id] = event.data["value"] = value
-            self.data["values"].append(evt_val_data)
         else:
             value.receive_event(event)
             event.data["value"] = value
-            self.data["values"][self.value_data_idx(value_id)].update(evt_val_data)
-
-        node_val_data = self.data["values"][self.value_data_idx(value_id)]
-        if "newValue" in evt_val_data:
-            node_val_data["value"] = evt_val_data["newValue"]
-
-        node_val_data.pop("newValue", None)
-        node_val_data.pop("prevValue", None)
 
     def handle_value_removed(self, event: Event) -> None:
         """Process a node value removed event."""
         value_id = _get_value_id_str_from_dict(self, event.data["args"])
         event.data["value"] = self.values.pop(value_id)
-        self.data["values"].pop(self.value_data_idx(value_id))
 
     def handle_value_notification(self, event: Event) -> None:
         """Process a node value notification event."""
@@ -942,25 +1058,27 @@ class Node(EventBase):
 
     def handle_notification(self, event: Event) -> None:
         """Process a node notification event."""
-        command_class = CommandClass(event.data["ccId"])
-        if command_class == CommandClass.NOTIFICATION:
-            event.data["notification"] = NotificationNotification(
-                self, cast(NotificationNotificationDataType, event.data)
-            )
-        elif command_class == CommandClass.SWITCH_MULTILEVEL:
-            event.data["notification"] = MultilevelSwitchNotification(
-                self, cast(MultilevelSwitchNotificationDataType, event.data)
-            )
-        elif command_class == CommandClass.ENTRY_CONTROL:
-            event.data["notification"] = EntryControlNotification(
-                self, cast(EntryControlNotificationDataType, event.data)
-            )
-        elif command_class == CommandClass.POWERLEVEL:
-            event.data["notification"] = PowerLevelNotification(
-                self, cast(PowerLevelNotificationDataType, event.data)
-            )
-        else:
-            _LOGGER.info("Unhandled notification command class: %s", command_class.name)
+        match command_class := CommandClass(event.data["ccId"]):
+            case CommandClass.NOTIFICATION:
+                event.data["notification"] = NotificationNotification(
+                    self, cast(NotificationNotificationDataType, event.data)
+                )
+            case CommandClass.SWITCH_MULTILEVEL:
+                event.data["notification"] = MultilevelSwitchNotification(
+                    self, cast(MultilevelSwitchNotificationDataType, event.data)
+                )
+            case CommandClass.ENTRY_CONTROL:
+                event.data["notification"] = EntryControlNotification(
+                    self, cast(EntryControlNotificationDataType, event.data)
+                )
+            case CommandClass.POWERLEVEL:
+                event.data["notification"] = PowerLevelNotification(
+                    self, cast(PowerLevelNotificationDataType, event.data)
+                )
+            case _:
+                _LOGGER.info(
+                    "Unhandled notification command class: %s", command_class.name
+                )
 
     def handle_firmware_update_progress(self, event: Event) -> None:
         """Process a node firmware update progress event."""
@@ -983,3 +1101,5 @@ class Node(EventBase):
         event.data["statistics_updated"] = self._statistics = NodeStatistics(
             self.client, statistics
         )
+        if last_seen := statistics.get("lastSeen"):
+            self.data["lastSeen"] = last_seen
